@@ -1,8 +1,12 @@
+# coding: utf-8
+
 from cms.plugins.text.cms_plugins import TextPlugin
 from .models import Footnote
 from django.utils.translation import ugettext_lazy as _
 from django.conf import settings
+from .utils import get_footnotes_for_page
 from cms.plugin_pool import plugin_pool
+from .utils import delete_cache_key
 
 
 class FootnotePlugin(TextPlugin):
@@ -11,7 +15,6 @@ class FootnotePlugin(TextPlugin):
     render_template = 'cmsplugin_footnote/footnote_symbol.html'
     text_enabled = True
     admin_preview = False
-    change_form_template = 'cms/plugins/text_plugin_change_form.html'
 
     def get_editor_widget(self, request, plugins):
         plugins.remove(FootnotePlugin)
@@ -24,15 +27,15 @@ class FootnotePlugin(TextPlugin):
     def render(self, context, instance, placeholder_name):
         context = super(FootnotePlugin, self).render(context, instance,
                                                      placeholder_name)
-        page = context['request'].current_page
-        placeholder = page.placeholders.get(slot=placeholder_name)
-        plugins = placeholder.cmsplugin_set \
-                             .filter(plugin_type='FootnotePlugin') \
-                             .order_by('tree_id', 'rght')
-        footnote_plugins = [p.get_plugin_instance()[0] for p in plugins]
-        context['counter'] = footnote_plugins.index(instance) + 1
-        context['placeholder_name'] = placeholder_name
+        request = context['request']
+        page = request.current_page
+        footnotes = get_footnotes_for_page(request, page)
+        context['counter'] = footnotes.index(instance) + 1
         return context
 
+    def save_model(self, *args, **kwargs):
+        super(FootnotePlugin, self).save_model(*args, **kwargs)
+        delete_cache_key(self.placeholder.page)
+
 
 plugin_pool.register_plugin(FootnotePlugin)
